@@ -255,7 +255,7 @@ const attributes = [
   {
     name: 'cite',
     tagName: 'blockquote',
-    overrideStringValue: 'http://reactjs.com/',
+    overrideStringValue: 'https://reactjs.com/',
   },
   {name: 'class', read: getAttribute('class')},
   {name: 'classID', tagName: 'object', read: getAttribute('classid')},
@@ -453,6 +453,11 @@ const attributes = [
     tagName: 'video',
     read: getProperty('disablepictureinpicture'),
   },
+  {
+    name: 'disableRemotePlayback',
+    tagName: 'video',
+    read: getProperty('disableremoteplayback'),
+  },
   {
     name: 'display',
     tagName: 'svg',
@@ -551,6 +556,11 @@ const attributes = [
     tagName: 'animate',
     read: getSVGAttribute('end'),
   },
+  {
+    name: 'enterKeyHint',
+    tagName: 'input',
+    read: getProperty('enterKeyHint'),
+  },
   {
     name: 'exponent',
     read: getSVGProperty('exponent'),
