@@ -5,6 +5,4 @@
  * LICENSE file in the root directory of this source tree.
  */
 
-'use strict';
-
-module.exports = require('./src/index');
+export * from './src/index';
