@@ -13,11 +13,11 @@
 //
 // Changes to search state may impact tree state.
 // For example, updating the selected search result also updates the tree's selected value.
-// Search does not fundamanetally change the tree though.
+// Search does not fundamentally change the tree though.
 // It is also possible to update the selected tree value independently.
 //
 // Changes to owners state mask search and tree values.
-// When owners statck is not empty, search is temporarily disabnled,
+// When owners stack is not empty, search is temporarily disabled,
 // and tree values (e.g. num elements, selected element) are masked.
 // Both tree and search values are restored when the owners stack is cleared.
 //
@@ -49,6 +49,7 @@ import type {Element} from './types';
 export type StateContext = {|
   // Tree
   numElements: number,
+  ownerSubtreeLeafElementID: number | null,
   selectedElementID: number | null,
   selectedElementIndex: number | null,
 
@@ -92,15 +93,27 @@ type ACTION_SELECT_ELEMENT_BY_ID = {|
 type ACTION_SELECT_NEXT_ELEMENT_IN_TREE = {|
   type: 'SELECT_NEXT_ELEMENT_IN_TREE',
 |};
+type ACTION_SELECT_NEXT_SIBLING_IN_TREE = {|
+  type: 'SELECT_NEXT_SIBLING_IN_TREE',
+|};
+type ACTION_SELECT_OWNER = {|
+  type: 'SELECT_OWNER',
+  payload: number,
+|};
 type ACTION_SELECT_PARENT_ELEMENT_IN_TREE = {|
   type: 'SELECT_PARENT_ELEMENT_IN_TREE',
 |};
 type ACTION_SELECT_PREVIOUS_ELEMENT_IN_TREE = {|
   type: 'SELECT_PREVIOUS_ELEMENT_IN_TREE',
 |};
-type ACTION_SELECT_OWNER = {|
-  type: 'SELECT_OWNER',
-  payload: number,
+type ACTION_SELECT_PREVIOUS_SIBLING_IN_TREE = {|
+  type: 'SELECT_PREVIOUS_SIBLING_IN_TREE',
+|};
+type ACTION_SELECT_OWNER_LIST_NEXT_ELEMENT_IN_TREE = {|
+  type: 'SELECT_OWNER_LIST_NEXT_ELEMENT_IN_TREE',
+|};
+type ACTION_SELECT_OWNER_LIST_PREVIOUS_ELEMENT_IN_TREE = {|
+  type: 'SELECT_OWNER_LIST_PREVIOUS_ELEMENT_IN_TREE',
 |};
 type ACTION_SET_SEARCH_TEXT = {|
   type: 'SET_SEARCH_TEXT',
@@ -119,9 +132,13 @@ type Action =
   | ACTION_SELECT_ELEMENT_AT_INDEX
   | ACTION_SELECT_ELEMENT_BY_ID
   | ACTION_SELECT_NEXT_ELEMENT_IN_TREE
+  | ACTION_SELECT_NEXT_SIBLING_IN_TREE
+  | ACTION_SELECT_OWNER
   | ACTION_SELECT_PARENT_ELEMENT_IN_TREE
   | ACTION_SELECT_PREVIOUS_ELEMENT_IN_TREE
-  | ACTION_SELECT_OWNER
+  | ACTION_SELECT_PREVIOUS_SIBLING_IN_TREE
+  | ACTION_SELECT_OWNER_LIST_NEXT_ELEMENT_IN_TREE
+  | ACTION_SELECT_OWNER_LIST_PREVIOUS_ELEMENT_IN_TREE
   | ACTION_SET_SEARCH_TEXT
   | ACTION_UPDATE_INSPECTED_ELEMENT_ID;
 
@@ -140,6 +157,7 @@ TreeDispatcherContext.displayName = 'TreeDispatcherContext';
 type State = {|
   // Tree
   numElements: number,
+  ownerSubtreeLeafElementID: number | null,
   selectedElementID: number | null,
   selectedElementIndex: number | null,
 
@@ -157,7 +175,13 @@ type State = {|
 |};
 
 function reduceTreeState(store: Store, state: State, action: Action): State {
-  let {numElements, ownerID, selectedElementIndex, selectedElementID} = state;
+  let {
+    numElements,
+    ownerSubtreeLeafElementID,
+    selectedElementIndex,
+    selectedElementID,
+  } = state;
+  const ownerID = state.ownerID;
 
   let lookupIDForIndex = true;
 
@@ -186,6 +210,8 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
         }
         break;
       case 'SELECT_CHILD_ELEMENT_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
         if (selectedElementIndex !== null) {
           const selectedElement = store.getElementAtIndex(
             ((selectedElementIndex: any): number),
@@ -204,9 +230,13 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
         }
         break;
       case 'SELECT_ELEMENT_AT_INDEX':
+        ownerSubtreeLeafElementID = null;
+
         selectedElementIndex = (action: ACTION_SELECT_ELEMENT_AT_INDEX).payload;
         break;
       case 'SELECT_ELEMENT_BY_ID':
+        ownerSubtreeLeafElementID = null;
+
         // Skip lookup in this case; it would be redundant.
         // It might also cause problems if the specified element was inside of a (not yet expanded) subtree.
         lookupIDForIndex = false;
@@ -218,6 +248,8 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
             : store.getIndexOfElementID(selectedElementID);
         break;
       case 'SELECT_NEXT_ELEMENT_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
         if (
           selectedElementIndex === null ||
           selectedElementIndex + 1 >= numElements
@@ -227,12 +259,80 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
           selectedElementIndex++;
         }
         break;
-      case 'SELECT_PARENT_ELEMENT_IN_TREE':
+      case 'SELECT_NEXT_SIBLING_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
         if (selectedElementIndex !== null) {
           const selectedElement = store.getElementAtIndex(
             ((selectedElementIndex: any): number),
           );
-          if (selectedElement !== null && selectedElement.parentID !== null) {
+          if (selectedElement !== null && selectedElement.parentID !== 0) {
+            const parent = store.getElementByID(selectedElement.parentID);
+            if (parent !== null) {
+              const {children} = parent;
+              const selectedChildIndex = children.indexOf(selectedElement.id);
+              const nextChildID =
+                selectedChildIndex < children.length - 1
+                  ? children[selectedChildIndex + 1]
+                  : children[0];
+              selectedElementIndex = store.getIndexOfElementID(nextChildID);
+            }
+          }
+        }
+        break;
+      case 'SELECT_OWNER_LIST_NEXT_ELEMENT_IN_TREE':
+        if (selectedElementIndex !== null) {
+          if (
+            ownerSubtreeLeafElementID !== null &&
+            ownerSubtreeLeafElementID !== selectedElementID
+          ) {
+            const leafElement = store.getElementByID(ownerSubtreeLeafElementID);
+            if (leafElement !== null) {
+              let currentElement = leafElement;
+              while (currentElement !== null) {
+                if (currentElement.ownerID === selectedElementID) {
+                  selectedElementIndex = store.getIndexOfElementID(
+                    currentElement.id,
+                  );
+                  break;
+                } else if (currentElement.ownerID !== 0) {
+                  currentElement = store.getElementByID(currentElement.ownerID);
+                }
+              }
+            }
+          }
+        }
+        break;
+      case 'SELECT_OWNER_LIST_PREVIOUS_ELEMENT_IN_TREE':
+        if (selectedElementIndex !== null) {
+          if (ownerSubtreeLeafElementID === null) {
+            // If this is the first time we're stepping through the owners tree,
+            // pin the current component as the owners list leaf.
+            // This will enable us to step back down to this component.
+            ownerSubtreeLeafElementID = selectedElementID;
+          }
+
+          const selectedElement = store.getElementAtIndex(
+            ((selectedElementIndex: any): number),
+          );
+          if (selectedElement !== null && selectedElement.ownerID !== 0) {
+            const ownerIndex = store.getIndexOfElementID(
+              selectedElement.ownerID,
+            );
+            if (ownerIndex !== null) {
+              selectedElementIndex = ownerIndex;
+            }
+          }
+        }
+        break;
+      case 'SELECT_PARENT_ELEMENT_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
+        if (selectedElementIndex !== null) {
+          const selectedElement = store.getElementAtIndex(
+            ((selectedElementIndex: any): number),
+          );
+          if (selectedElement !== null && selectedElement.parentID !== 0) {
             const parentIndex = store.getIndexOfElementID(
               selectedElement.parentID,
             );
@@ -243,12 +343,35 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
         }
         break;
       case 'SELECT_PREVIOUS_ELEMENT_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
         if (selectedElementIndex === null || selectedElementIndex === 0) {
           selectedElementIndex = numElements - 1;
         } else {
           selectedElementIndex--;
         }
         break;
+      case 'SELECT_PREVIOUS_SIBLING_IN_TREE':
+        ownerSubtreeLeafElementID = null;
+
+        if (selectedElementIndex !== null) {
+          const selectedElement = store.getElementAtIndex(
+            ((selectedElementIndex: any): number),
+          );
+          if (selectedElement !== null && selectedElement.parentID !== 0) {
+            const parent = store.getElementByID(selectedElement.parentID);
+            if (parent !== null) {
+              const {children} = parent;
+              const selectedChildIndex = children.indexOf(selectedElement.id);
+              const nextChildID =
+                selectedChildIndex > 0
+                  ? children[selectedChildIndex - 1]
+                  : children[children.length - 1];
+              selectedElementIndex = store.getIndexOfElementID(nextChildID);
+            }
+          }
+        }
+        break;
       default:
         // React can bailout of no-op updates.
         return state;
@@ -270,6 +393,7 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
     ...state,
 
     numElements,
+    ownerSubtreeLeafElementID,
     selectedElementIndex,
     selectedElementID,
   };
@@ -277,13 +401,13 @@ function reduceTreeState(store: Store, state: State, action: Action): State {
 
 function reduceSearchState(store: Store, state: State, action: Action): State {
   let {
-    ownerID,
     searchIndex,
     searchResults,
     searchText,
     selectedElementID,
     selectedElementIndex,
   } = state;
+  const ownerID = state.ownerID;
 
   const prevSearchIndex = searchIndex;
   const prevSearchText = searchText;
@@ -452,10 +576,8 @@ function reduceOwnersState(store: Store, state: State, action: Action): State {
     selectedElementIndex,
     ownerID,
     ownerFlatTree,
-    searchIndex,
-    searchResults,
-    searchText,
   } = state;
+  const {searchIndex, searchResults, searchText} = state;
 
   let prevSelectedElementIndex = selectedElementIndex;
 
@@ -654,8 +776,12 @@ function TreeContextController({
         case 'SELECT_ELEMENT_BY_ID':
         case 'SELECT_CHILD_ELEMENT_IN_TREE':
         case 'SELECT_NEXT_ELEMENT_IN_TREE':
+        case 'SELECT_NEXT_SIBLING_IN_TREE':
+        case 'SELECT_OWNER_LIST_NEXT_ELEMENT_IN_TREE':
+        case 'SELECT_OWNER_LIST_PREVIOUS_ELEMENT_IN_TREE':
         case 'SELECT_PARENT_ELEMENT_IN_TREE':
         case 'SELECT_PREVIOUS_ELEMENT_IN_TREE':
+        case 'SELECT_PREVIOUS_SIBLING_IN_TREE':
         case 'SELECT_OWNER':
         case 'UPDATE_INSPECTED_ELEMENT_ID':
         case 'SET_SEARCH_TEXT':
@@ -688,6 +814,7 @@ function TreeContextController({
   const [state, dispatch] = useReducer(reducer, {
     // Tree
     numElements: store.numElements,
+    ownerSubtreeLeafElementID: null,
     selectedElementID:
       defaultSelectedElementID == null ? null : defaultSelectedElementID,
     selectedElementIndex:
@@ -734,7 +861,7 @@ function TreeContextController({
       prevSelectedElementID.current = state.selectedElementID;
 
       if (state.selectedElementID !== null) {
-        let element = store.getElementByID(state.selectedElementID);
+        const element = store.getElementByID(state.selectedElementID);
         if (element !== null && element.parentID > 0) {
           store.toggleIsCollapsed(element.parentID, false);
         }
