@@ -54,7 +54,7 @@ describe('ReactDOMInput', () => {
     expect(() => {
       ReactDOM.render(<input type="text" value={0} />, container);
     }).toErrorDev(
-      'Failed prop type: You provided a `value` prop to a form field without an `onChange` handler.',
+      'Warning: You provided a `value` prop to a form field without an `onChange` handler.',
     );
   });
 
@@ -62,7 +62,7 @@ describe('ReactDOMInput', () => {
     expect(() => {
       ReactDOM.render(<input type="text" value="" />, container);
     }).toErrorDev(
-      'Failed prop type: You provided a `value` prop to a form field without an `onChange` handler.',
+      'Warning: You provided a `value` prop to a form field without an `onChange` handler.',
     );
   });
 
@@ -70,7 +70,7 @@ describe('ReactDOMInput', () => {
     expect(() => {
       ReactDOM.render(<input type="text" value="0" />, container);
     }).toErrorDev(
-      'Failed prop type: You provided a `value` prop to a form field without an `onChange` handler.',
+      'Warning: You provided a `value` prop to a form field without an `onChange` handler.',
     );
   });
 
@@ -78,7 +78,7 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="checkbox" checked={false} />, container),
     ).toErrorDev(
-      'Failed prop type: You provided a `checked` prop to a form field without an `onChange` handler.',
+      'Warning: You provided a `checked` prop to a form field without an `onChange` handler.',
     );
   });
 
@@ -95,7 +95,7 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'Failed prop type: You provided a `checked` prop to a form field without an `onChange` handler. ' +
+      'Warning: You provided a `checked` prop to a form field without an `onChange` handler. ' +
         'This will render a read-only field. If the field should be mutable use `defaultChecked`. ' +
         'Otherwise, set either `onChange` or `readOnly`.',
     );
@@ -115,13 +115,17 @@ describe('ReactDOMInput', () => {
     ReactDOM.render(<input type="checkbox" checked={undefined} />, container);
   });
 
+  it('should not warn with value and onInput handler', () => {
+    ReactDOM.render(<input value="..." onInput={() => {}} />, container);
+  });
+
   it('should properly control a value even if no event listener exists', () => {
     let node;
 
     expect(() => {
       node = ReactDOM.render(<input type="text" value="lion" />, container);
     }).toErrorDev(
-      'Failed prop type: You provided a `value` prop to a form field without an `onChange` handler.',
+      'Warning: You provided a `value` prop to a form field without an `onChange` handler.',
     );
 
     setUntrackedValue.call(node, 'giraffe');
@@ -179,6 +183,7 @@ describe('ReactDOMInput', () => {
     // bypass the lazy event attachment system so we won't actually test this.
     dispatchEventOnNode(instance.a, 'input');
     dispatchEventOnNode(instance.a, 'blur');
+    dispatchEventOnNode(instance.a, 'focusout');
 
     expect(instance.a.value).toBe('giraffe');
     expect(instance.switchedFocus).toBe(true);
@@ -379,7 +384,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should display `defaultValue` of number 0', () => {
-    let stub = <input type="text" defaultValue={0} />;
+    const stub = <input type="text" defaultValue={0} />;
     const node = ReactDOM.render(stub, container);
 
     expect(node.getAttribute('value')).toBe('0');
@@ -411,14 +416,14 @@ describe('ReactDOMInput', () => {
   });
 
   it('should display "true" for `defaultValue` of `true`', () => {
-    let stub = <input type="text" defaultValue={true} />;
+    const stub = <input type="text" defaultValue={true} />;
     const node = ReactDOM.render(stub, container);
 
     expect(node.value).toBe('true');
   });
 
   it('should display "false" for `defaultValue` of `false`', () => {
-    let stub = <input type="text" defaultValue={false} />;
+    const stub = <input type="text" defaultValue={false} />;
     const node = ReactDOM.render(stub, container);
 
     expect(node.value).toBe('false');
@@ -475,8 +480,7 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="text" defaultValue="1" />, container),
     ).toErrorDev(
-      'A component is changing a controlled input of type ' +
-        'text to be uncontrolled.',
+      'A component is changing a controlled input to be uncontrolled.',
     );
     expect(node.value).toBe('0');
   });
@@ -681,6 +685,7 @@ describe('ReactDOMInput', () => {
       expect(node.hasAttribute('value')).toBe(false);
     } else {
       dispatchEventOnNode(node, 'blur');
+      dispatchEventOnNode(node, 'focusout');
 
       expect(node.value).toBe('0.0');
       expect(node.getAttribute('value')).toBe('0.0');
@@ -854,8 +859,7 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'A component is changing a controlled input of type ' +
-        'submit to be uncontrolled.',
+      'A component is changing a controlled input to be uncontrolled.',
     );
 
     const node = container.firstChild;
@@ -874,8 +878,7 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'A component is changing a controlled input of type ' +
-        'reset to be uncontrolled.',
+      'A component is changing a controlled input to be uncontrolled.',
     );
 
     const node = container.firstChild;
@@ -883,7 +886,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should set a value on a submit input', () => {
-    let stub = <input type="submit" value="banana" />;
+    const stub = <input type="submit" value="banana" />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -891,7 +894,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should not set an undefined value on a submit input', () => {
-    let stub = <input type="submit" value={undefined} />;
+    const stub = <input type="submit" value={undefined} />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -904,7 +907,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should not set an undefined value on a reset input', () => {
-    let stub = <input type="reset" value={undefined} />;
+    const stub = <input type="reset" value={undefined} />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -917,7 +920,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should not set a null value on a submit input', () => {
-    let stub = <input type="submit" value={null} />;
+    const stub = <input type="submit" value={null} />;
     expect(() => {
       ReactDOM.render(stub, container);
     }).toErrorDev('`value` prop on `input` should not be null');
@@ -932,7 +935,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should not set a null value on a reset input', () => {
-    let stub = <input type="reset" value={null} />;
+    const stub = <input type="reset" value={null} />;
     expect(() => {
       ReactDOM.render(stub, container);
     }).toErrorDev('`value` prop on `input` should not be null');
@@ -947,7 +950,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should set a value on a reset input', () => {
-    let stub = <input type="reset" value="banana" />;
+    const stub = <input type="reset" value="banana" />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -955,7 +958,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should set an empty string value on a submit input', () => {
-    let stub = <input type="submit" value="" />;
+    const stub = <input type="submit" value="" />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -963,7 +966,7 @@ describe('ReactDOMInput', () => {
   });
 
   it('should set an empty string value on a reset input', () => {
-    let stub = <input type="reset" value="" />;
+    const stub = <input type="reset" value="" />;
     ReactDOM.render(stub, container);
     const node = container.firstChild;
 
@@ -1168,7 +1171,7 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'Warning: Failed prop type: You provided a `value` prop to a form ' +
+      'Warning: You provided a `value` prop to a form ' +
         'field without an `onChange` handler. This will render a read-only ' +
         'field. If the field should be mutable use `defaultValue`. ' +
         'Otherwise, set either `onChange` or `readOnly`.\n' +
@@ -1225,7 +1228,7 @@ describe('ReactDOMInput', () => {
         '(specify either the checked prop, or the defaultChecked prop, but not ' +
         'both). Decide between using a controlled or uncontrolled input ' +
         'element and remove one of these props. More info: ' +
-        'https://fb.me/react-controlled-components',
+        'https://reactjs.org/link/controlled-components',
     );
     ReactDOM.unmountComponentAtNode(container);
 
@@ -1252,7 +1255,7 @@ describe('ReactDOMInput', () => {
         '(specify either the value prop, or the defaultValue prop, but not ' +
         'both). Decide between using a controlled or uncontrolled input ' +
         'element and remove one of these props. More info: ' +
-        'https://fb.me/react-controlled-components',
+        'https://reactjs.org/link/controlled-components',
     );
     ReactDOM.unmountComponentAtNode(container);
 
@@ -1268,10 +1271,11 @@ describe('ReactDOMInput', () => {
     );
     ReactDOM.render(stub, container);
     expect(() => ReactDOM.render(<input type="text" />, container)).toErrorDev(
-      'Warning: A component is changing a controlled input of type text to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1286,10 +1290,11 @@ describe('ReactDOMInput', () => {
     ).toErrorDev([
       '`value` prop on `input` should not be null. ' +
         'Consider using an empty string to clear the component or `undefined` for uncontrolled components',
-      'Warning: A component is changing a controlled input of type text to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     ]);
   });
@@ -1305,10 +1310,11 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type text to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1319,10 +1325,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="text" value="controlled" />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type text to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1336,10 +1343,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="text" value="controlled" />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type text to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1352,10 +1360,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="checkbox" />, container),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type checkbox to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1368,10 +1377,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="checkbox" checked={null} />, container),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type checkbox to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1387,10 +1397,11 @@ describe('ReactDOMInput', () => {
         container,
       ),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type checkbox to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1401,10 +1412,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="checkbox" checked={true} />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type checkbox to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1415,10 +1427,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="checkbox" checked={true} />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type checkbox to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1427,10 +1440,11 @@ describe('ReactDOMInput', () => {
     const stub = <input type="radio" checked={true} onChange={emptyFunction} />;
     ReactDOM.render(stub, container);
     expect(() => ReactDOM.render(<input type="radio" />, container)).toErrorDev(
-      'Warning: A component is changing a controlled input of type radio to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1441,10 +1455,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="radio" checked={null} />, container),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type radio to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1455,10 +1470,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="radio" defaultChecked={true} />, container),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type radio to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1469,10 +1485,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="radio" checked={true} />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type radio to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1483,10 +1500,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="radio" checked={true} />, container),
     ).toErrorDev(
-      'Warning: A component is changing an uncontrolled input of type radio to be controlled. ' +
-        'Input elements should not switch from uncontrolled to controlled (or vice versa). ' +
+      'Warning: A component is changing an uncontrolled input to be controlled. ' +
+        'This is likely caused by the value changing from undefined to ' +
+        'a defined value, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1535,10 +1553,11 @@ describe('ReactDOMInput', () => {
     expect(() =>
       ReactDOM.render(<input type="radio" value="value" />, container),
     ).toErrorDev(
-      'Warning: A component is changing a controlled input of type radio to be uncontrolled. ' +
-        'Input elements should not switch from controlled to uncontrolled (or vice versa). ' +
+      'Warning: A component is changing a controlled input to be uncontrolled. ' +
+        'This is likely caused by the value changing from a defined to ' +
+        'undefined, which should not happen. ' +
         'Decide between using a controlled or uncontrolled input ' +
-        'element for the lifetime of the component. More info: https://fb.me/react-controlled-components\n' +
+        'element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components\n' +
         '    in input (at **)',
     );
   });
@@ -1621,15 +1640,39 @@ describe('ReactDOMInput', () => {
     const originalCreateElement = document.createElement;
     spyOnDevAndProd(document, 'createElement').and.callFake(function(type) {
       const el = originalCreateElement.apply(this, arguments);
-      let value = '';
+      const getDefaultValue = Object.getOwnPropertyDescriptor(
+        HTMLInputElement.prototype,
+        'defaultValue',
+      ).get;
+      const setDefaultValue = Object.getOwnPropertyDescriptor(
+        HTMLInputElement.prototype,
+        'defaultValue',
+      ).set;
+      const getValue = Object.getOwnPropertyDescriptor(
+        HTMLInputElement.prototype,
+        'value',
+      ).get;
+      const setValue = Object.getOwnPropertyDescriptor(
+        HTMLInputElement.prototype,
+        'value',
+      ).set;
       if (type === 'input') {
-        Object.defineProperty(el, 'value', {
+        Object.defineProperty(el, 'defaultValue', {
           get: function() {
-            return value;
+            return getDefaultValue.call(this);
+          },
+          set: function(val) {
+            log.push(`node.defaultValue = ${strify(val)}`);
+            setDefaultValue.call(this, val);
+          },
+        });
+        Object.defineProperty(el, 'value', {
+          get: function() {
+            return getValue.call(this);
           },
           set: function(val) {
-            value = '' + val;
             log.push(`node.value = ${strify(val)}`);
+            setValue.call(this, val);
           },
         });
         spyOnDevAndProd(el, 'setAttribute').and.callFake(function(name, val) {
@@ -1644,12 +1687,18 @@ describe('ReactDOMInput', () => {
     if (disableInputAttributeSyncing) {
       expect(log).toEqual([
         'node.setAttribute("type", "date")',
-        'node.setAttribute("value", "1980-01-01")',
+        'node.defaultValue = "1980-01-01"',
+        // TODO: it's possible this reintroduces the bug because we don't assign `value` at all.
+        // Need to check this on mobile Safari and Chrome.
       ]);
     } else {
       expect(log).toEqual([
         'node.setAttribute("type", "date")',
+        // value must be assigned before defaultValue. This fixes an issue where the
+        // visually displayed value of date inputs disappears on mobile Safari and Chrome:
+        // https://github.com/facebook/react/issues/7233
         'node.value = "1980-01-01"',
+        'node.defaultValue = "1980-01-01"',
       ]);
     }
   });
@@ -1724,6 +1773,7 @@ describe('ReactDOMInput', () => {
       // be the only way to remove focus in JSDOM
       node.blur();
       dispatchEventOnNode(node, 'blur');
+      dispatchEventOnNode(node, 'focusout');
 
       if (disableInputAttributeSyncing) {
         expect(node.value).toBe('2');
@@ -1749,6 +1799,7 @@ describe('ReactDOMInput', () => {
       // be the only way to remove focus in JSDOM
       node.blur();
       dispatchEventOnNode(node, 'blur');
+      dispatchEventOnNode(node, 'focusout');
 
       expect(node.getAttribute('value')).toBe('1');
     });
@@ -1768,6 +1819,7 @@ describe('ReactDOMInput', () => {
       // be the only way to remove focus in JSDOM
       node.blur();
       dispatchEventOnNode(node, 'blur');
+      dispatchEventOnNode(node, 'focusout');
 
       expect(node.getAttribute('value')).toBe('1');
     });
@@ -1803,8 +1855,7 @@ describe('ReactDOMInput', () => {
 
     it('reverts the value attribute to the initial value', () => {
       expect(renderInputWithStringThenWithUndefined).toErrorDev(
-        'Input elements should not switch from controlled to ' +
-          'uncontrolled (or vice versa).',
+        'A component is changing a controlled input to be uncontrolled.',
       );
       if (disableInputAttributeSyncing) {
         expect(input.getAttribute('value')).toBe(null);
@@ -1815,8 +1866,7 @@ describe('ReactDOMInput', () => {
 
     it('preserves the value property', () => {
       expect(renderInputWithStringThenWithUndefined).toErrorDev(
-        'Input elements should not switch from controlled to ' +
-          'uncontrolled (or vice versa).',
+        'A component is changing a controlled input to be uncontrolled.',
       );
       expect(input.value).toBe('latest');
     });
@@ -1855,8 +1905,7 @@ describe('ReactDOMInput', () => {
         '`value` prop on `input` should not be null. ' +
           'Consider using an empty string to clear the component ' +
           'or `undefined` for uncontrolled components.',
-        'Input elements should not switch from controlled ' +
-          'to uncontrolled (or vice versa).',
+        'A component is changing a controlled input to be uncontrolled.',
       ]);
       if (disableInputAttributeSyncing) {
         expect(input.hasAttribute('value')).toBe(false);
@@ -1870,8 +1919,7 @@ describe('ReactDOMInput', () => {
         '`value` prop on `input` should not be null. ' +
           'Consider using an empty string to clear the component ' +
           'or `undefined` for uncontrolled components.',
-        'Input elements should not switch from controlled ' +
-          'to uncontrolled (or vice versa).',
+        'A component is changing a controlled input to be uncontrolled.',
       ]);
       expect(input.value).toBe('latest');
     });
