@@ -21,10 +21,23 @@ describe('ReactUpdates', () => {
     React = require('react');
     ReactDOM = require('react-dom');
     ReactTestUtils = require('react-dom/test-utils');
-    act = ReactTestUtils.act;
+    act = ReactTestUtils.unstable_concurrentAct;
     Scheduler = require('scheduler');
   });
 
+  // Note: This is based on a similar component we use in www. We can delete
+  // once the extra div wrapper is no longer necessary.
+  function LegacyHiddenDiv({children, mode}) {
+    return (
+      <div hidden={mode === 'hidden'}>
+        <React.unstable_LegacyHidden
+          mode={mode === 'hidden' ? 'unstable-defer-without-hiding' : mode}>
+          {children}
+        </React.unstable_LegacyHidden>
+      </div>
+    );
+  }
+
   it('should batch state when updating state twice', () => {
     let updateCount = 0;
 
@@ -526,7 +539,6 @@ describe('ReactUpdates', () => {
 
     const bContainer = document.createElement('div');
 
-    let a;
     let b;
 
     let aUpdated = false;
@@ -560,7 +572,7 @@ describe('ReactUpdates', () => {
       }
     }
 
-    a = ReactTestUtils.renderIntoDocument(<A />);
+    const a = ReactTestUtils.renderIntoDocument(<A />);
     ReactDOM.unstable_batchedUpdates(function() {
       a.setState({x: 1});
       b.setState({x: 1});
@@ -733,11 +745,8 @@ describe('ReactUpdates', () => {
       }
     }
 
-    let x;
-    let y;
-
-    x = ReactTestUtils.renderIntoDocument(<X />);
-    y = ReactTestUtils.renderIntoDocument(<Y />);
+    const x = ReactTestUtils.renderIntoDocument(<X />);
+    const y = ReactTestUtils.renderIntoDocument(<Y />);
     expect(ReactDOM.findDOMNode(x).textContent).toBe('0');
 
     y.forceUpdate();
@@ -1102,7 +1111,7 @@ describe('ReactUpdates', () => {
     'in legacy mode, updates in componentWillUpdate and componentDidUpdate ' +
       'should both flush in the immediately subsequent commit',
     () => {
-      let ops = [];
+      const ops = [];
       class Foo extends React.Component {
         state = {a: false, b: false};
         UNSAFE_componentWillUpdate(_, nextState) {
@@ -1145,7 +1154,7 @@ describe('ReactUpdates', () => {
     'in legacy mode, updates in componentWillUpdate and componentDidUpdate ' +
       '(on a sibling) should both flush in the immediately subsequent commit',
     () => {
-      let ops = [];
+      const ops = [];
       class Foo extends React.Component {
         state = {a: false};
         UNSAFE_componentWillUpdate(_, nextState) {
@@ -1213,7 +1222,7 @@ describe('ReactUpdates', () => {
   );
 
   it('uses correct base state for setState inside render phase', () => {
-    let ops = [];
+    const ops = [];
 
     class Foo extends React.Component {
       state = {step: 0};
@@ -1236,7 +1245,7 @@ describe('ReactUpdates', () => {
   });
 
   it('does not re-render if state update is null', () => {
-    let container = document.createElement('div');
+    const container = document.createElement('div');
 
     let instance;
     let ops = [];
@@ -1291,85 +1300,62 @@ describe('ReactUpdates', () => {
     expect(ops).toEqual(['Foo', 'Bar', 'Baz']);
   });
 
-  it.experimental(
-    'delays sync updates inside hidden subtrees in Concurrent Mode',
-    () => {
-      const container = document.createElement('div');
+  // @gate experimental
+  it('delays sync updates inside hidden subtrees in Concurrent Mode', () => {
+    const container = document.createElement('div');
 
-      function Baz() {
-        Scheduler.unstable_yieldValue('Baz');
-        return <p>baz</p>;
-      }
+    function Baz() {
+      Scheduler.unstable_yieldValue('Baz');
+      return <p>baz</p>;
+    }
 
-      let setCounter;
-      function Bar() {
-        const [counter, _setCounter] = React.useState(0);
-        setCounter = _setCounter;
-        Scheduler.unstable_yieldValue('Bar');
-        return <p>bar {counter}</p>;
-      }
+    let setCounter;
+    function Bar() {
+      const [counter, _setCounter] = React.useState(0);
+      setCounter = _setCounter;
+      Scheduler.unstable_yieldValue('Bar');
+      return <p>bar {counter}</p>;
+    }
 
-      function Foo() {
-        Scheduler.unstable_yieldValue('Foo');
-        React.useEffect(() => {
-          Scheduler.unstable_yieldValue('Foo#effect');
-        });
-        return (
-          <div>
-            <div hidden={true}>
-              <Bar />
-            </div>
-            <Baz />
-          </div>
-        );
-      }
-
-      const root = ReactDOM.createRoot(container);
-      let hiddenDiv;
-      act(() => {
-        root.render(<Foo />);
-        if (__DEV__) {
-          expect(Scheduler).toFlushAndYieldThrough([
-            'Foo',
-            'Foo',
-            'Baz',
-            'Foo#effect',
-          ]);
-        } else {
-          expect(Scheduler).toFlushAndYieldThrough([
-            'Foo',
-            'Baz',
-            'Foo#effect',
-          ]);
-        }
-        hiddenDiv = container.firstChild.firstChild;
-        expect(hiddenDiv.hidden).toBe(true);
-        expect(hiddenDiv.innerHTML).toBe('');
-        // Run offscreen update
-        if (__DEV__) {
-          expect(Scheduler).toFlushAndYield(['Bar', 'Bar']);
-        } else {
-          expect(Scheduler).toFlushAndYield(['Bar']);
-        }
-        expect(hiddenDiv.hidden).toBe(true);
-        expect(hiddenDiv.innerHTML).toBe('<p>bar 0</p>');
+    function Foo() {
+      Scheduler.unstable_yieldValue('Foo');
+      React.useEffect(() => {
+        Scheduler.unstable_yieldValue('Foo#effect');
       });
+      return (
+        <div>
+          <LegacyHiddenDiv mode="hidden">
+            <Bar />
+          </LegacyHiddenDiv>
+          <Baz />
+        </div>
+      );
+    }
 
-      ReactDOM.flushSync(() => {
-        setCounter(1);
-      });
-      // Should not flush yet
-      expect(hiddenDiv.innerHTML).toBe('<p>bar 0</p>');
-
+    const root = ReactDOM.unstable_createRoot(container);
+    let hiddenDiv;
+    act(() => {
+      root.render(<Foo />);
+      expect(Scheduler).toFlushAndYieldThrough(['Foo', 'Baz', 'Foo#effect']);
+      hiddenDiv = container.firstChild.firstChild;
+      expect(hiddenDiv.hidden).toBe(true);
+      expect(hiddenDiv.innerHTML).toBe('');
       // Run offscreen update
-      if (__DEV__) {
-        expect(Scheduler).toFlushAndYield(['Bar', 'Bar']);
-      } else {
-        expect(Scheduler).toFlushAndYield(['Bar']);
-      }
-      expect(hiddenDiv.innerHTML).toBe('<p>bar 1</p>');
-    },
-  );
+      expect(Scheduler).toFlushAndYield(['Bar']);
+      expect(hiddenDiv.hidden).toBe(true);
+      expect(hiddenDiv.innerHTML).toBe('<p>bar 0</p>');
+    });
+
+    ReactDOM.flushSync(() => {
+      setCounter(1);
+    });
+    // Should not flush yet
+    expect(hiddenDiv.innerHTML).toBe('<p>bar 0</p>');
+
+    // Run offscreen update
+    expect(Scheduler).toFlushAndYield(['Bar']);
+    expect(hiddenDiv.innerHTML).toBe('<p>bar 1</p>');
+  });
 
   it('can render ridiculously large number of roots without triggering infinite update loop error', () => {
     class Foo extends React.Component {
@@ -1625,7 +1611,7 @@ describe('ReactUpdates', () => {
 
       let error = null;
       let stack = null;
-      let originalConsoleError = console.error;
+      const originalConsoleError = console.error;
       console.error = (e, s) => {
         error = e;
         stack = s;
@@ -1640,7 +1626,7 @@ describe('ReactUpdates', () => {
               Scheduler.unstable_clearYields();
             }
             expect(error).toContain('Warning: Maximum update depth exceeded.');
-            expect(stack).toContain('in NonTerminating');
+            expect(stack).toContain(' NonTerminating');
             // rethrow error to prevent going into an infinite loop when act() exits
             throw error;
           });
@@ -1700,7 +1686,7 @@ describe('ReactUpdates', () => {
   }
 
   if (__DEV__) {
-    it('should properly trace interactions within batched udpates', () => {
+    it('should properly trace interactions within batched updates', () => {
       const SchedulerTracing = require('scheduler/tracing');
 
       let expectedInteraction;
