@@ -13,35 +13,58 @@ import typeof * as EventListenerType from '../EventListener';
 import typeof * as EventListenerShimType from './EventListener-www';
 
 export function addEventBubbleListener(
-  element: Element,
+  target: EventTarget,
   eventType: string,
   listener: Function,
-): void {
-  EventListenerWWW.listen(element, eventType, listener);
+) {
+  return EventListenerWWW.listen(target, eventType, listener);
 }
 
 export function addEventCaptureListener(
-  element: Element,
+  target: EventTarget,
   eventType: string,
   listener: Function,
-): void {
-  EventListenerWWW.capture(element, eventType, listener);
+) {
+  return EventListenerWWW.capture(target, eventType, listener);
 }
 
 export function addEventCaptureListenerWithPassiveFlag(
-  element: Element,
+  target: EventTarget,
   eventType: string,
   listener: Function,
   passive: boolean,
-): void {
-  EventListenerWWW.captureWithPassiveFlag(
-    element,
+) {
+  return EventListenerWWW.captureWithPassiveFlag(
+    target,
     eventType,
     listener,
     passive,
   );
 }
 
+export function addEventBubbleListenerWithPassiveFlag(
+  target: EventTarget,
+  eventType: string,
+  listener: Function,
+  passive: boolean,
+) {
+  return EventListenerWWW.bubbleWithPassiveFlag(
+    target,
+    eventType,
+    listener,
+    passive,
+  );
+}
+
+export function removeEventListener(
+  target: EventTarget,
+  eventType: string,
+  listener: Function,
+  capture: boolean,
+) {
+  listener.remove();
+}
+
 // Flow magic to verify the exports of this file match the original version.
 // eslint-disable-next-line no-unused-vars
 type Check<_X, Y: _X, X: Y = _X> = null;
