@@ -10,7 +10,11 @@
 import type {ReactNodeList} from 'shared/ReactTypes';
 import type {Writable} from 'stream';
 
-import {createRequest, startWork, startFlowing} from 'react-server/inline.dom';
+import {
+  createRequest,
+  startWork,
+  startFlowing,
+} from 'react-server/src/ReactFizzServer';
 
 function createDrainHandler(destination, request) {
   return () => startFlowing(request);
@@ -20,11 +24,9 @@ function pipeToNodeWritable(
   children: ReactNodeList,
   destination: Writable,
 ): void {
-  let request = createRequest(children, destination);
+  const request = createRequest(children, destination);
   destination.on('drain', createDrainHandler(destination, request));
   startWork(request);
 }
 
-export default {
-  pipeToNodeWritable,
-};
+export {pipeToNodeWritable};
