@@ -7,6 +7,4 @@
  * @flow
  */
 
-'use strict';
-
-module.exports = require('./src/dom/Focus');
+export * from './src/dom/create-event-handle/Focus';
