@@ -7,8 +7,6 @@
 
 import invariant from 'shared/invariant';
 
-import {enableNativeTargetAsInstance} from 'shared/ReactFeatureFlags';
-
 const instanceCache = new Map();
 const instanceProps = new Map();
 
@@ -26,23 +24,14 @@ function getInstanceFromTag(tag) {
 }
 
 function getTagFromInstance(inst) {
-  if (enableNativeTargetAsInstance) {
-    let nativeInstance = inst.stateNode;
-    let tag = nativeInstance._nativeTag;
-    if (tag === undefined) {
-      nativeInstance = nativeInstance.canonical;
-      tag = nativeInstance._nativeTag;
-    }
-    invariant(tag, 'All native instances should have a tag.');
-    return nativeInstance;
-  } else {
-    let tag = inst.stateNode._nativeTag;
-    if (tag === undefined) {
-      tag = inst.stateNode.canonical._nativeTag;
-    }
-    invariant(tag, 'All native instances should have a tag.');
-    return tag;
+  let nativeInstance = inst.stateNode;
+  let tag = nativeInstance._nativeTag;
+  if (tag === undefined) {
+    nativeInstance = nativeInstance.canonical;
+    tag = nativeInstance._nativeTag;
   }
+  invariant(tag, 'All native instances should have a tag.');
+  return nativeInstance;
 }
 
 export {
