@@ -7,9 +7,9 @@
  * @flow
  */
 
-export type HookEffectTag = number;
+export type HookFlags = number;
 
-export const NoEffect = /*  */ 0b000;
+export const NoFlags = /*  */ 0b000;
 
 // Represents whether effect should fire.
 export const HasEffect = /* */ 0b001;
