@@ -9,10 +9,11 @@
 
 export type TypeOfMode = number;
 
-export const NoMode = 0b0000;
-export const StrictMode = 0b0001;
+export const NoMode = 0b00000;
+export const StrictMode = 0b00001;
 // TODO: Remove BlockingMode and ConcurrentMode by reading from the root
 // tag instead
-export const BlockingMode = 0b0010;
-export const ConcurrentMode = 0b0100;
-export const ProfileMode = 0b1000;
+export const BlockingMode = 0b00010;
+export const ConcurrentMode = 0b00100;
+export const ProfileMode = 0b01000;
+export const DebugTracingMode = 0b10000;
