@@ -23,6 +23,10 @@ export {
   useDebugValue,
   useLayoutEffect,
   useMemo,
+  useMutableSource,
+  useMutableSource as unstable_useMutableSource,
+  createMutableSource,
+  createMutableSource as unstable_createMutableSource,
   useReducer,
   useRef,
   useState,
@@ -30,6 +34,7 @@ export {
   Profiler,
   StrictMode,
   Suspense,
+  unstable_LegacyHidden,
   createElement,
   cloneElement,
   isValidElement,
@@ -37,18 +42,20 @@ export {
   __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
   // exposeConcurrentModeAPIs
   useTransition,
+  useTransition as unstable_useTransition,
   useDeferredValue,
+  useDeferredValue as unstable_useDeferredValue,
+  startTransition,
+  startTransition as unstable_startTransition,
   SuspenseList,
-  unstable_withSuspenseConfig,
+  SuspenseList as unstable_SuspenseList,
   // enableBlocksAPI
   block,
-  // enableDeprecatedFlareAPI
-  DEPRECATED_useResponder,
-  DEPRECATED_createResponder,
+  block as unstable_block,
   // enableScopeAPI
-  unstable_createScope,
-  // enableJSXTransformAPI
-  jsx,
-  jsxs,
-  jsxDEV,
+  unstable_Scope,
+  unstable_useOpaqueIdentifier,
+  // enableDebugTracing
+  unstable_DebugTracingMode,
 } from './src/React';
+export {jsx, jsxs, jsxDEV} from './src/jsx/ReactJSX';
