@@ -18,6 +18,7 @@
 let PropTypes;
 let React;
 let ReactDOM;
+let ReactDOMServer;
 let ReactTestUtils;
 
 describe('ReactContextValidator', () => {
@@ -27,6 +28,7 @@ describe('ReactContextValidator', () => {
     PropTypes = require('prop-types');
     React = require('react');
     ReactDOM = require('react-dom');
+    ReactDOMServer = require('react-dom/server');
     ReactTestUtils = require('react-dom/test-utils');
   });
 
@@ -261,7 +263,7 @@ describe('ReactContextValidator', () => {
 
     class Component extends React.Component {
       render() {
-        return <TestContext.Provider />;
+        return <TestContext.Provider value={undefined} />;
       }
     }
 
@@ -671,4 +673,39 @@ describe('ReactContextValidator', () => {
       'Warning: ComponentB: Function components do not support contextType.',
     );
   });
+
+  it('should honor a displayName if set on the context type', () => {
+    const Context = React.createContext(null);
+    Context.displayName = 'MyContextType';
+    function Validator() {
+      return null;
+    }
+    Validator.propTypes = {dontPassToSeeErrorStack: PropTypes.bool.isRequired};
+
+    expect(() => {
+      ReactDOMServer.renderToStaticMarkup(
+        <Context.Provider>
+          <Context.Consumer>{() => <Validator />}</Context.Consumer>
+        </Context.Provider>,
+      );
+    }).toErrorDev(
+      'Warning: Failed prop type: The prop `dontPassToSeeErrorStack` is marked as required in `Validator`, but its value is `undefined`.\n' +
+        '    in Validator (at **)',
+    );
+  });
+
+  it('warns if displayName is set on the consumer type', () => {
+    const Context = React.createContext(null);
+
+    expect(() => {
+      Context.Consumer.displayName = 'IgnoredName';
+    }).toWarnDev(
+      'Warning: Setting `displayName` on Context.Consumer has no effect. ' +
+        "You should set it directly on the context with Context.displayName = 'IgnoredName'.",
+      {withoutStack: true},
+    );
+
+    // warning is deduped by Context so subsequent setting is fine
+    Context.Consumer.displayName = 'ADifferentName';
+  });
 });
