@@ -12,33 +12,27 @@ import ReactCurrentDispatcher from '../ReactCurrentDispatcher';
 import ReactCurrentOwner from '../ReactCurrentOwner';
 import ReactDebugCurrentFrame from '../ReactDebugCurrentFrame';
 import IsSomeRendererActing from '../IsSomeRendererActing';
+import ReactCurrentBatchConfig from '../ReactCurrentBatchConfig';
 
 const ReactSharedInternals = {
   ReactCurrentDispatcher,
   ReactCurrentOwner,
   IsSomeRendererActing,
+  ReactCurrentBatchConfig,
   // Used by renderers to avoid bundling object-assign twice in UMD bundles:
   assign,
+
+  // Re-export the schedule API(s) for UMD bundles.
+  // This avoids introducing a dependency on a new UMD global in a minor update,
+  // Since that would be a breaking change (e.g. for all existing CodeSandboxes).
+  // This re-export is only required for UMD bundles;
+  // CJS bundles use the shared NPM package.
+  Scheduler,
+  SchedulerTracing,
 };
 
 if (__DEV__) {
-  Object.assign(ReactSharedInternals, {
-    // These should not be included in production.
-    ReactDebugCurrentFrame,
-    // Shim for React DOM 16.0.0 which still destructured (but not used) this.
-    // TODO: remove in React 17.0.
-    ReactComponentTreeHook: {},
-  });
+  ReactSharedInternals.ReactDebugCurrentFrame = ReactDebugCurrentFrame;
 }
 
-// Re-export the schedule API(s) for UMD bundles.
-// This avoids introducing a dependency on a new UMD global in a minor update,
-// Since that would be a breaking change (e.g. for all existing CodeSandboxes).
-// This re-export is only required for UMD bundles;
-// CJS bundles use the shared NPM package.
-Object.assign(ReactSharedInternals, {
-  Scheduler,
-  SchedulerTracing,
-});
-
 export default ReactSharedInternals;
