@@ -11,15 +11,18 @@ const theme = require('../theme');
 const run = async ({cwd, packages, tags}) => {
   clear();
 
-  if (tags.length === 1) {
+  if (tags.length === 0) {
+    console.error('Expected at least one tag.');
+    process.exit(1);
+  } else if (tags.length === 1) {
     console.log(
-      theme`{spinnerSuccess ✓} You are about the publish the following packages under the tag {tag ${tags}}`
+      theme`{spinnerSuccess ✓} You are about the publish the following packages under the tag {tag ${tags}}:`
     );
   } else {
     console.log(
       theme`{spinnerSuccess ✓} You are about the publish the following packages under the tags {tag ${tags.join(
         ', '
-      )}}`
+      )}}:`
     );
   }
 
