@@ -14,6 +14,7 @@ module.exports = {
     WeakMap: true,
     WeakSet: true,
     Uint16Array: true,
+    Reflect: true,
     // Vendor specific
     MSApp: true,
     __REACT_DEVTOOLS_GLOBAL_HOOK__: true,
@@ -29,9 +30,19 @@ module.exports = {
     Int32Array: true,
     ArrayBuffer: true,
 
+    TaskController: true,
+
     // Flight
     Uint8Array: true,
     Promise: true,
+
+    // Flight Webpack
+    __webpack_chunk_load__: true,
+    __webpack_require__: true,
+
+    // jest
+    expect: true,
+    jest: true,
   },
   parserOptions: {
     ecmaVersion: 5,
